try:
    from .custom_nodes import *
    from .nodes import *
    from .logs import get_logger
except ImportError:
    from custom_nodes import *
    from nodes import *
    from logs import get_logger

import boto3

logger = get_logger(__name__)



"""
1. check for already existing operation data
2. get relations of the operation
3. find the dependent operations to call
4. read the target dependent operation
5. generate JMESpath selectors with the relations
6. search with JMESpath to get raw_parameters
7. generate real api parameters from raw_parameters
8. read the current operation
"""
if __name__ == '__main__':
    session = boto3.session.Session()
    all_service_names = get_all_available_services(session)
    success_count, failure_count, non_req_count = 0,0, 0
    
    service_node = ServiceNode('ec2', session)
    
    # service_node = ServiceNode('accessanalyzer', session)
    # service_reader = ServiceReader(service_node)

    # aa = service_reader.read_operation('PasswordData', 'GetPasswordData')
    # aa = service_reader.read_operation('AccessAnalyzer', 'GetAccessAnalyzer')
    # aa = service_reader.read_operation('NetworkInsightsAccessScopeAnalyses', 'DescribeNetworkInsightsAccessScopeAnalyses')
    # aa = service_reader.read_operation('ChangeSet', 'ListChangeSets')
    
    # aa

    # aa = service_reader.read_operation('Instances', 'DescribeInstances')
    # aa = service_reader.read_operation('SecurityGroups', 'DescribeSecurityGroups')
    # if aa:
    #     aa
    # for service_name in all_service_names:
        # continue
    # for service_name in []:
    for service_name in ['ec2']:
        service_node = ServiceNode(service_name, session)
        service_node
        # print(colored(service_name, 'yellow'))
        # service_model = service_node.get_service_model()
        # for shape_name in service_model.shape_names:
        #     shape = service_model.shape_for(shape_name)
        #     shape
        #     pattern = shape.metadata.get('pattern', False)
        #     if shape.metadata.get('pattern', False):
        #         print('\t', colored(shape_name, 'blue'),colored(pattern, 'green'))
    
    
    
    
    #     # if not service_name=='amplifybackend':
    #     #     continue
        print('-'*50)
        print('#'*10, colored(service_name, 'yellow'), '#'*10)
        # resource_nodes = service_node.get_resource_nodes()
        relation_map = service_node.get_relation_map().get_relations_map()
        service_reader = ServiceReader(service_node)
        
        # a = relation_map.get_relations_map()
        # a
        # a = service_reader.recursively_get_dependent_operations('GetAnalyzer')
        # a
        # continue
        read_operation_names = service_node.get_read_operation_names()
        for r_ops_name in read_operation_names:
            verb, *resource_name_tokens = camel_case_split(r_ops_name)
            resource_name = ''.join(resource_name_tokens)
            # if resource_name!='amplifybackend':
            #     continue
            x = service_reader.get_dependent_operations(r_ops_name)
            if x :
                print(x)
            a = service_reader.read_operation(resource_name, r_ops_name)
    # #         # x = relation_map.find_best_relations_for_operations_parameters(r_ops_name)
            if a:
                a
    #         if x:
    #             if len(x) == 1:
    #                 non_req_count+= 1
    #             success_count += 1 
    #         else:
    #             failure_count += 1
            
            
    #         # get_required_parameter_names_from_operation_name()
    #         print_flag = False
    #         if type(x)==dict:
    #             for _item in x.values():
    #                 if type(_item) in [dict, list]:
    #                     print_flag = True
    #                     break
    #         if print_flag:
    #             print(json.dumps(x, indent=2, default=str))
    #         if x:
    #             print( json.dumps(x, indent=2, default=str))
    #         else:
    #             print(colored('\t'+r_ops_name+'   FALSE', 'red'))
    #     print('SUCCESS', success_count)
    #     print('FAILURE', failure_count)
    #     print('NON_REQUIRED_PARAMS', non_req_count)
    #     print('TOTAL', success_count+failure_count)


from typing import Optional
import typer

try:
    from .utils import *
    from .logs import get_logger, get_rich_console
    from .nodes import ServiceNode
    from .reader import ServiceReader
    from .custom_nodes import *
except ImportError:
    from utils import *
    from logs import get_logger, get_rich_console
    from nodes import ServiceNode
    from reader import ServiceReader
    from custom_nodes import *

import boto3
from rich.columns import Columns
from rich.panel import Panel
from rich.console import Group
from rich.layout import Layout
from rich.pretty import Pretty
import os

def _create_boto_session():
    profile_name = os.environ.get('AWS_PROFILE', False)
    region_name = os.environ.get('AWS_REGION', False)
    _kwargs_dict = {}
    if profile_name:
        _kwargs_dict['profile_name']=profile_name
    if region_name:
        _kwargs_dict['region_name']=region_name
    session = boto3.session.Session(**_kwargs_dict)
    return session 

console = get_rich_console()
logger = get_logger(__name__)
session = _create_boto_session()

app = typer.Typer(no_args_is_help=True)
aws_app = typer.Typer() # no_args_is_help=True
app.add_typer(aws_app, name="aws")
# aws_ls_app = typer.Typer(no_args_is_help=True)
# aws_app.add_typer(aws_ls_app, name="ls")

def _get_available_service_node_names():
    return get_all_available_services(session)

def generate_service_node_completion_items():
    service_names = _get_available_service_node_names(session)
    return service_names

def _complete_service_name(incomplete: str):
    service_names = _get_available_service_node_names()
    if not incomplete:
        for name in service_names:
            yield name
    else:
        for name in service_names:
            if name.startswith(incomplete):
                yield name



def _complete_resource_node_name(ctx: typer.Context, incomplete: str):
    service = ctx.params.get("service") or False
    print('HEYHEYEY')
    print(service)   
    if not service:
        return []
    service_node = ServiceNode(service, session)
    if not service_node:
        return []
    resource_nodes = service_node.get_resource_nodes()
    if not resource_nodes:
        return []
    
    if incomplete:
        for _rn in resource_nodes:
            _rn_name = _rn.name
            if _rn_name.startswith(incomplete):
                if _rn.get_all_required_parameter_names():
                    _rn_name = f"[bold]{_rn_name}[/]"
                yield _rn_name
    else:
        for _rn in resource_nodes:
            _rn_name = _rn.name
            if _rn.get_all_required_parameter_names():
                _rn_name = f"[bold]{_rn_name}[/]"
            yield _rn_name
    # if not service:
        
    # for name, help_text in valid_completion_items:
    #     if name.startswith(incomplete) and name not in names:
    #         yield (name, help_text)


@aws_app.command('ls')
def _cli_ls_command(
        service: Optional[str] = typer.Argument(None, show_default=False,help='The AWS service name', autocompletion=_complete_service_name),
        resource_node: Optional[str] = typer.Argument(None, show_default=False, help='The AWS Resource Node', autocompletion=_complete_resource_node_name),
    ):
    """
    Lists and details AWS Services/ResourceNodes.
    TODO: Document the functions here for --help option
    """
    available_service_names = _get_available_service_node_names()
    if not service and not resource_node:
        # nothing is given
        console.print(Columns(available_service_names, equal=True, expand=True))
        return
    
    if service and not resource_node:
        # we only have service name
        
        if service not in available_service_names:
            similar_service_names = ifind_similar_names_in_list(service, available_service_names)
            if similar_service_names:
                console.print(f"[yellow]Found Similar Service Names:")
                console.print(Columns(similar_service_names, equal=True, expand=True))
                return
            else:
                raise typer.Exit(f"Invalid service name: {service}. Please pick a proper one.")
            
        service_node = ServiceNode(service, session)
        resource_nodes = service_node.get_resource_nodes()
        
        resource_node_names = []
        for _rn in resource_nodes:
            _rn_name = _rn.name
            if _rn.get_all_required_parameter_names():
                _rn_name = f"[bold]{_rn_name}[/]"
            resource_node_names.append(_rn_name)
        
        resource_node_name_as_columns =  Columns(resource_node_names, equal=True, expand=True)
        console.print(resource_node_name_as_columns)
        return
        
    elif service and resource_node:
        # we got both options filled
        service_node = ServiceNode(service, session)
        resource_nodes = service_node.get_resource_nodes()
        resource_node_names = [ _rn.name for _rn in resource_nodes]
        resource_node_obj = service_node.get_resource_node_by_name(resource_node)
        if not resource_node_obj:
            
            similar_resource_names = ifind_similar_names_in_list(resource_node, resource_node_names)
            if similar_resource_names:
                console.print(f"[yellow]Found Similar Resource Node Names:")
                console.print(Columns(similar_resource_names, equal=True, expand=True))
                return 
            else:
                raise typer.Exit(f"Invalid Resource Node name: [bold]{resource_node}[/] for Service: [bold]{service}[/].")
        operations_panel = service_node._get_operation_details_panel(resource_node_obj.name)
        console.print(operations_panel)

        return    


@aws_app.command('read')
def _cli_read_command(
        service: Optional[str] = typer.Argument(None, show_default=False,help='The AWS service name', autocompletion=_complete_service_name),
        resource_node: Optional[str] = typer.Argument(None, show_default=False, help='The AWS Resource Node', autocompletion=_complete_resource_node_name),
    ):
    available_service_names = _get_available_service_node_names()
    if not service and not resource_node:
        _cli_ls_command(service, resource_node)
        console.print(Panel(f"[bold]Please pick one of the AWS Services", title="[red][bold]ERROR"))
        return
    if service and not resource_node:
        _cli_ls_command(service, resource_node)
        console.print(Panel(f"[bold]Please pick one of the Resource Nodes from [green]{service}[/] Service", title="[red][bold]ERROR"))
        return
    elif service and resource_node:
        service_node = ServiceNode(service, session)
        service_reader = ServiceReader(service_node)
        read = service_reader.read_resource_node(resource_node)
        console.print(Pretty(read))
        # console.print(Panel(read, title=f"[green][bold]{resource_node} RESPONSE"))
        return
    
    
def run_app():
    app()
    
if __name__ == "__main__":
    # typer.run(main)
    app()
    
    







# import typer

# valid_completion_items = [
#     ("Camila", "The reader of books."),
#     ("Carlos", "The writer of scripts."),
#     ("Sebastian", "The type hints guy."),
# ]


# def complete_name(incomplete: str):
#     completion = []
#     for name, help_text in valid_completion_items:
#         if name.startswith(incomplete):
#             completion_item = (name, help_text)
#             completion.append(completion_item)
#     return completion


# def main(
#     name: str = typer.Option(
#         "World", help="The name to say hi to.", autocompletion=complete_name
#     )
# ):
#     print(f"Hello {name}")


# if __name__ == "__main__":
#     typer.run(main)



import logging
from rich.logging import RichHandler
from rich.console import Console


_console = Console()


def get_rich_console() -> Console:
    """Returns the global defined `rich.console.Console` object for common use. 

    Returns:
        rich.console.Console: rich console object
    """
    return _console

def supress_other_module_logs() -> None:
    """Sets Python modules log levels to `logging.CRITICAL` to supress them."""
    _supress_module_names = ('boto', 'urllib3', 's3transfer', 'boto3', 'botocore', 'nose')
    for _logger_name in logging.Logger.manager.loggerDict.keys():
        if _logger_name in _supress_module_names:
            logging.getLogger(_logger_name).setLevel(logging.CRITICAL)
            
            
def get_logger(name: str) -> logging.Logger:
    """Logger creation with RichHandler. 

    Args:
        name (str): Logger name. Usually `__name__`.

    Returns:
        logging.Logger: Logger obj
    """
    supress_other_module_logs()
    logging.basicConfig(
        # level="NOTSET",
        level="DEBUG",
        format="%(message)s",
        datefmt="[%X]",
        handlers=[RichHandler(rich_tracebacks=True, markup=True, console=_console)]
    )
    return logging.getLogger(name)
